@@ -28,6 +28,12 @@ TUBE_TYPE_MAPPING = {
 }
 
 
+def flatten(llist):
+    if isinstance(llist, list):
+        return sum(llist, [])
+    return llist
+
+
 class TubeMessage:
 
     @staticmethod
@@ -376,20 +382,21 @@ class Tube:
 
 class TubeNode:
 
-    def __init__(self, *, schema=None):
+    def __init__(self, *, schema=None, warning_not_mach_topic=True):
         self.logger = logging.getLogger(self.__class__.__name__)
         self._tubes = TopicMatcher()
         self._callbacks = TopicMatcher()
         if schema:
             self.parse_schema(schema)
         self._stop_main_loop = False
+        self.warning_not_mach_topic = warning_not_mach_topic
 
     @property
     def tubes(self) -> [Tube]:
         """
         returns a list of all registered tubes
         """
-        return self._tubes.values()
+        return flatten(self._tubes.values())
 
     def connect(self):
         """
@@ -414,17 +421,25 @@ class TubeNode:
                 tube = Tube(**tube_info)
                 self.register_tube(tube, tube_info.get('topics', []))
 
-    def get_tube_by_topic(self, topic: str) -> Tube:
+    def get_tube_by_topic(self, topic: str, types=None) -> Tube:
         """
         returns the Tube which is assigned to topic.
+        Optional: we can specify a type of tube.
         """
-        return self._tubes.match(topic)
+        res = self._tubes.match(topic)
+        if types:
+            res = [t for t in res if t.tube_type in types]
+        if not res:
+            return None
+        if isinstance(res, list):
+            res = res.pop()
+        return res
 
     def get_tube_by_name(self, name: str) -> Tube:
         """
         returns the Tube with the name
         """
-        tubes = self._tubes.values()
+        tubes = flatten(self._tubes.values())
         for tube in tubes:
             if tube.name == name:
                 return tube
@@ -437,60 +452,57 @@ class TubeNode:
         if isinstance(topics, str):
             topics = [topics]
         for topic in topics:
-            if self._tubes.get_topic(topic):
-                self.logger.warning(f"The tube '{tube.name}' overrides "
-                                    f"the exist topic: {topic}")
+            tubes = self._tubes.get_topic(topic)
+            if tubes:
+                self.logger.info(f"The tube '{tube.name}' overrides "
+                                 f"the exist topic: {topic}")
+                tubes.append(tube)
             else:
                 self.logger.debug(f"The tube '{tube.name}' was registered to "
                                   f"the topic: {topic}")
-            self._tubes.set_topic(topic, tube)
+                tubes = [tube, ]
+            self._tubes.set_topic(topic, tubes)
 
     def get_callback_by_topic(self, topic: str) -> Callable:
-        return self._callbacks.match(topic)
+        res = self._callbacks.match(topic)
+        return res
 
     def send(self, topic: str, payload=None, tube=None):
         if not tube:
-            tube = self.get_tube_by_topic(topic)
-        if not tube:
-            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assign '
-                                         f'to any Tube.')
+            tube = self.get_tube_by_topic(topic, [zmq.DEALER])
+            if not tube:
+                raise TubeTopicNotConfigured(f'The topic "{topic}" is not '
+                                             f'assigned to any Tube for '
+                                             f'dealer.')
         tube.send(topic, payload)
 
     async def request(self, topic: str, payload=None, timeout=30) \
             -> TubeMessage:
-        tube = self.get_tube_by_topic(topic)
+        tube = self.get_tube_by_topic(topic, [zmq.REQ])
         if not tube:
-            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assign '
-                                         f'to any Tube.')
+            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assigned '
+                                         f'to any Tube for request.')
         res = await tube.request(topic, payload, timeout)
         return res
 
     def publish(self, topic: str, payload=None):
-        tube = self.get_tube_by_topic(topic)
+        tube = self.get_tube_by_topic(topic, [zmq.PUB])
         if not tube:
-            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assign '
-                                         f'to any Tube.')
-        if tube.tube_type != zmq.PUB:
-            raise TubeMethodNotSupported(
-                f"The tube '{tube.name}' (type: '{tube.tube_type_name}') "
-                f"can not publish message."
-            )
-        self.send(topic, payload)
+            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assigned '
+                                         f'to any Tube for publishing.')
+        self.send(topic, payload, tube)
 
     def subscribe(self, topic: str, fce: Callable):
-        tube = self.get_tube_by_topic(topic)
+        tube = self.get_tube_by_topic(topic, [zmq.SUB])
         if not tube:
-            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assign '
-                                         f'to any Tube.')
-        if tube.tube_type != zmq.SUB:
-            raise TubeMethodNotSupported(
-                f"The tube '{tube.name}' (type: '{tube.tube_type_name}') "
-                f"can not subscribe topic."
-            )
-        self.register_handler(topic, fce)
+            raise TubeTopicNotConfigured(f'The topic "{topic}" is not assigned '
+                                         f'to any Tube for subscribe.')
+        self.register_handler(topic, fce, tube=tube)
 
-    def register_handler(self, topic: str, fce: Callable):
-        tube = self.get_tube_by_topic(topic)
+    def register_handler(self, topic: str, fce: Callable, tube: Tube = None):
+        if not tube:
+            tube = self.get_tube_by_topic(topic, [zmq.SUB, zmq.REP,
+                                                  zmq.ROUTER, zmq.DEALER])
         if not tube:
             self.logger.warning(f"This topic '{topic}' does not have any "
                                 f"assigned tube. This callback will be "
@@ -539,10 +551,11 @@ class TubeNode:
                 request = await tube.receive_data(raw_socket=raw_socket)
                 callbacks = self.get_callback_by_topic(request.topic)
                 if not callbacks:
-                    self.logger.warning(
-                        f"Incoming message does not match any topic, "
-                        f"it is ignored (topic: {request.topic})"
-                    )
+                    if self.warning_not_mach_topic:
+                        self.logger.warning(
+                            f"Incoming message does not match any topic, "
+                            f"it is ignored (topic: {request.topic})"
+                        )
                     continue
                 # self.logger.debug(
                 #     f"Incoming message for tube '{tube.name}'")
